# Hye-Shik Chang <16 Feb 2002>
# $Id: __init__.py,v 1.2 2002/02/17 13:48:00 perky Exp $

__aliases__ = {
    'korean.cp949':         ('cp949', 'ms949', '949', 'uhc'),
    'korean.euc_kr':        ('euc_kr', 'ksc5601', 'ksc5601_1987'),
    'korean.iso_2022_kr':   ('iso_2022_kr',),
    'korean.johab':         ('johab',),
    'korean.qwerty2bul':    ('qwerty2bul', '2bul'),
    'korean.unijohab':      ('unijohab', 'unicode_johab', 'macjohab'),
}

from encodings.aliases import aliases

for enc, als in __aliases__.items():
    for aliasenam in als:
        aliases[aliasenam] = enc

del aliases, als, enc, aliasenam
