# Hye-Shik Chang <16 Feb 2002>
# originally written by Tamito KAJIYAMA
#
# $Id: cp949.py,v 1.1.1.1 2002/02/16 00:51:35 perky Exp $

from korean.python.euc_kr import *

from korean.mappings import uhc
encmap_hangul.update(uhc.encoding_map)
decmap_hangul.update(uhc.decoding_map)
del uhc

def getaliases():
    return ('cp949', 'uhc', 'ms949',)
